@@ -2,8 +2,9 @@
 /*
   Class struct of libisoburn.
 
-  Copyright 2007 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
-             and Thomas Schmitt <scdbackup@gmx.net>
+  Copyright 2007 - 2012 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+                    and Thomas Schmitt <scdbackup@gmx.net>
+  Provided under GPL version 2 or later.
 */
 
 #ifndef Isoburn_includeD
@@ -11,7 +12,13 @@
 
 
 /* for uint8_t */
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 /* For emulated TOC of overwriteable media.
    Provides minimal info for faking a struct burn_toc_entry.
@@ -22,6 +29,8 @@ struct isoburn_toc_entry {
  int start_lba;
  int track_blocks;
 
+ char *volid; /* For caching a volume id from emulated toc on overwriteables */
+
  struct isoburn_toc_entry *next;
 };
 
@@ -33,11 +42,26 @@ int isoburn_toc_entry_new(struct isoburn_toc_entry **objpt,
 int isoburn_toc_entry_destroy(struct isoburn_toc_entry **o, int flag);
 
 
-/* Size of target_iso_head which is to be written during
-   isoburn_activate_session()
+/* Minimal size of target_iso_head which is to be written during
+   isoburn_activate_session().
+   Within this size there is everything that is needed for image access with
+   no partition offset. The actual target_iso_head buffer must be larger by
+   the evential partition offset.
 */
 #define Libisoburn_target_head_sizE (32*2048)
 
+
+/* Maximum number of appended partitions. Effectively usable number depends
+   on system area type.
+*/
+#define Libisoburn_max_appended_partitionS 8
+
+/*
+  Maximum length of a disc label text plus 1.
+*/
+#define Libisoburn_disc_label_sizE 129
+
+
 struct isoburn {
 
 
@@ -45,9 +69,9 @@ struct isoburn {
     Most isoburn calls will use a burn_drive as object handle */
  struct burn_drive *drive;
 
- /* -1= inappropriate media state detected
-    0= libburn multi-session media, resp. undecided yet
-    1= random access media */
+ /* -1= inappropriate medium state detected
+    0= libburn multi-session medium, resp. undecided yet
+    1= random access medium */
  int emulation_mode;
 
  /* Although rarely used, libburn can operate on several
@@ -89,6 +113,10 @@ struct isoburn {
  */
  enum burn_disc_status fabricated_disc_status;
 
+ /* To be set if read errors occured during media evaluation.
+ */
+ int media_read_error;
+
  /* Eventual emulated table of content read from the chain of ISO headers
     on overwriteable media.
  */ 
@@ -101,13 +129,21 @@ struct isoburn {
  int wrote_well;
 
  
- /* Buffered ISO head from media (should that become part of
-    ecma119_read_opts ?) */
- uint8_t target_iso_head[Libisoburn_target_head_sizE];
+ /* ISO head buffer to be filled by write run */
+ int target_iso_head_size;
+ uint8_t *target_iso_head;
+
+ /* The 2k offset which was read from a loaded image.
+ */
+ uint32_t loaded_partition_offset;
+ 
 
  /* Libisofs image context */
  IsoImage *image;
 
+ /* The start LBA of the image */
+ int image_start_lba;
+
  /* The block data source from which the existing image is read.
  */
  IsoDataSource *iso_data_source;
@@ -129,6 +165,9 @@ struct isoburn {
  void *msgs_submit_handle;  /* specific to application method */
  int msgs_submit_flag;      /* specific to application method */
 
+ /* Forwarding an image generation option to the burn wrapper */
+ int do_tao;
+ 
 };
 
 
@@ -182,7 +221,7 @@ int isoburn_invalidate_iso(struct isoburn *o, int flag);
                  with no emulation activated by isoburn.
     @param drive The drive to be searched for
     @param flag  unused yet
-    @return -1 unsuitable media, 0 generic media, 1 emulated media.
+    @return -1 unsuitable medium, 0 generic medium, 1 emulated medium.
 */
 int isoburn_find_emulator(struct isoburn **pt,
                           struct burn_drive *drive, int flag);
@@ -194,7 +233,7 @@ int isoburn_msgs_submit(struct isoburn *o, int error_code, char msg_text[],
                         int os_errno, char severity[], int flag);
 
 /** Set the start address for an emulated add-on session. The value will
-    be rounded up to the alignment necessary for the media. The aligned
+    be rounded up to the alignment necessary for the medium. The aligned
     value will be divided by 2048 and then put into  o->nwa .
     @param o     The isoburn object to be programmed.
     @param value The start address in bytes
@@ -219,12 +258,27 @@ int isoburn_get_msc2(struct isoburn *o,
 /** Get a data source suitable for read from a drive using burn_read_data()
     function.
     @param d drive to read from. Must be grabbed.
+    @param displacement will be added or subtracted to any block address
+    @param displacement_sign  +1 = add , -1= subtract , else keep unaltered
     @return the data source, NULL on error. Must be freed with libisofs
             iso_data_source_unref() function. Note: this doesn't release
             the drive.
 */
 IsoDataSource *
-isoburn_data_source_new(struct burn_drive *d);
+isoburn_data_source_new(struct burn_drive *d,
+                         uint32_t displacement, int displacement_sign,
+                         int cache_tiles, int tile_blocks);
+
+/** Default settings for above cache_tiles, tile_blocks in newly created
+    struct isoburn_read_opts.
+*/
+#define Libisoburn_default_cache_tileS 32
+#define Libisoburn_default_tile_blockS 32
+
+/** Maximum size of the cache in 2 kB blocks (1 GB)
+*/
+#define Libisoburn_cache_max_sizE (1024 * 512)
+
 
 /** Disable read capabilities of a data source which was originally created
     by isoburn_data_source_new(). After this any attempt to read will yield
@@ -240,6 +294,18 @@ isoburn_data_source_new(struct burn_drive *d);
 int isoburn_data_source_shutdown(IsoDataSource *src, int flag);
 
 
+/** Check whether the size of target_iso_head matches the given partition
+    offset. Eventually adjust size.
+*/
+int isoburn_adjust_target_iso_head(struct isoburn *o,
+                                   uint32_t offst, int flag);
+
+
+/** Initialize the root directory attributes of a freshly created image.
+*/
+int isoburn_root_defaults(IsoImage *image, int flag);
+
+
 /**
  * Options for image reading.
    (Comments here may be outdated. API getter/setter function descriptions
@@ -247,6 +313,9 @@ int isoburn_data_source_shutdown(IsoDataSource *src, int flag);
     minor correction only.)
  */
 struct isoburn_read_opts {
+    int cache_tiles; /* number of cache tiles */
+    int cache_tile_blocks;
+
     unsigned int norock:1; /*< Do not read Rock Ridge extensions */
     unsigned int nojoliet:1; /*< Do not read Joliet extensions */
     unsigned int noiso1999:1; /*< Do not read ISO 9660:1999 enhanced tree */
@@ -256,6 +325,12 @@ struct isoburn_read_opts {
     unsigned int noacl:1;    /* Do not read ACL from external file objects */
     unsigned int noea:1;     /* Do not read XFS-style EA from externals */
 
+    /* ts A90508 */
+    unsigned int noino:1;    /* Discard eventual PX inode numbers */
+
+    /* ts A90810 */
+    unsigned int nomd5:2;    /* Do not read eventual MD5 array */
+
     unsigned int preferjoliet:1; 
                 /*< When both Joliet and RR extensions are present, the RR
                  *  tree is used. If you prefer using Joliet, set this to 1. */
@@ -270,6 +345,15 @@ struct isoburn_read_opts {
      */
     char *input_charset;
 
+    /**
+     * Enable or disable methods to automatically choose an input charset.
+     * This eventually overrides input_charset.
+     *
+     * bit0= allow to set the input character set automatically from
+     *       attribute "isofs.cs" of root directory
+     */
+    int auto_input_charset;
+
     /* modified by the function isoburn_read_image */
     unsigned int hasRR:1; /*< It will be set to 1 if RR extensions are present,
                              to 0 if not. */
@@ -288,6 +372,9 @@ struct isoburn_read_opts {
     uint32_t size; /**< Will be filled with the size (in 2048 byte block) of 
                     *   the image, as reported in the PVM. */
     unsigned int pretend_blank:1; /* always create empty image */
+
+    uint32_t displacement;
+    int displacement_sign;
 };
 
 
@@ -301,15 +388,42 @@ struct isoburn_imgen_opts {
 
     /* Options for image generation */
 
+    int will_cancel :1;
+
     int level;  /**< ISO level to write at. */
     
     /** Which extensions to support. */
     unsigned int rockridge :1;
     unsigned int joliet :1;
     unsigned int iso1999 :1;
+    unsigned int hfsplus :1;
+    unsigned int fat :1;
+
+    /* Whether to mark suitable IsoNode as hardlinks in RRIP PX */
+    unsigned int hardlinks :1;
+
+    /* Write eventual AAIP info containing ACL and EA */
+    unsigned int aaip :1;
+
+    /* Produce and write a MD5 checksum of the whole session stream. */
+    unsigned int session_md5 :1;
+
+    /* Produce and write MD5 checksums for each single IsoFile.
+       See parameter "files" of iso_write_opts_set_record_md5().
+     */
+    unsigned int file_md5 :2;
+
+    /* On overwriteable media or random access files do not write the first
+       session to LBA 32, but rather to LBA 0 directly.
+     */
+    unsigned int no_emul_toc :1;
+
+    /* For empty files, symbolic links, and devices use the old ECMA-119 block
+       addresses in the range [0,31] rather than the address of the dedicated
+       empty block.
+    */
+    unsigned int old_empty :1;
 
-    /* ts A90121 */
-    unsigned int aaip:1;   /* Write eventual AAIP info containing ACL and EA */
 
     /* relaxed constraints */
 
@@ -318,17 +432,52 @@ struct isoburn_imgen_opts {
      * but it is supposed to work on most moderns systems. Use with caution.
      */
 
+    /*
+     * Extra Caution: This option breaks any assumptions about names that
+     *                are supported by ECMA-119 specifications.
+     * Omit any translation which would make a file name compliant to the
+     * ECMA-119 rules. This includes and exceeds omit_version_numbers,
+     * max_37_char_filenames, no_force_dots bit0, allow_lowercase.
+     */
+    unsigned int untranslated_name_len;
+
+    /*
+     * Convert directory names for ECMA-119 similar to other file names, but do
+     * not force a dot or add a version number.
+     * This violates ECMA-119 by allowing one "." and especially ISO level 1
+     * by allowing DOS style 8.3 names rather than only 8 characters.
+     * (mkisofs and its clones seem to do this violation.)
+     */
+    unsigned int allow_dir_id_ext :1;
+
     /**
      * Omit the version number (";1") at the end of the ISO-9660 identifiers.
      * Version numbers are usually not used.
+     *      bit0= omit version number with ECMA-119 and Joliet
+     *      bit1= omit version number with Joliet alone
      */
-    unsigned int omit_version_numbers :1;
+    unsigned int omit_version_numbers :2;
 
     /**
      * Allow ISO-9660 directory hierarchy to be deeper than 8 levels.
      */
     unsigned int allow_deep_paths :1;
 
+    /**
+     * If not allow_deep_paths is in effect, then it may become
+     * necessary to relocate directories so that no ECMA-119 file path
+     * has more than 8 components. These directories are grafted into either
+     * the root directory of the ISO image or into a dedicated relocation
+     * directory. For details see libisofs.h, iso_write_opts_set_rr_reloc().
+     */
+    char *rr_reloc_dir;       /* IsoNode name in root directory. NULL or
+                                 empty text means root itself. */
+    int rr_reloc_flags;       /* bit0= mark auto-created rr_reloc_dir by RE
+                                 bit1= not settable via API (used internally)
+                              */
+
+
+
     /**
      * Allow path in the ISO-9660 tree to have more than 255 characters.
      */
@@ -346,8 +495,10 @@ struct isoburn_imgen_opts {
      * ISO-9660 forces filenames to have a ".", that separates file name from
      * extension. libisofs adds it if original filename doesn't has one. Set
      * this to 1 to prevent this behavior
+     *      bit0= no forced dot with ECMA-119
+     *      bit1= no forced dot with Joliet
      */
-    unsigned int no_force_dots :1;
+    unsigned int no_force_dots :2;
 
     /**
      * Allow lowercase characters in ISO-9660 filenames. By default, only
@@ -361,11 +512,24 @@ struct isoburn_imgen_opts {
      */
     unsigned int allow_full_ascii :1;
 
+    /**
+     * Like allow_full_ascii, but only allowing 7-bit characters.
+     * Lowercase letters get mapped to uppercase if not allow_lowercase is set.
+     * Gets overridden if allow_full_ascii is enabled.
+     */
+    unsigned int allow_7bit_ascii :1;
+
     /**
      * Allow paths in the Joliet tree to have more than 240 characters.
      */
     unsigned int joliet_longer_paths :1;
 
+    /**
+     * Allow leaf names in the Joliet tree to have up to 103 characters
+     * rather than 64.
+     */
+    unsigned int joliet_long_names :1;
+
     /**
      * Store timestamps as GMT rather than in local time.
      */
@@ -381,8 +545,11 @@ struct isoburn_imgen_opts {
     /**
      * Store as ECMA-119 Directory Record timestamp the mtime
      * of the source rather than the image creation time.
+     * The same can be ordered for Joliet and ISO 9660:1999
      */
     unsigned int dir_rec_mtime :1;
+    unsigned int joliet_rec_mtime :1;
+    unsigned int iso1999_rec_mtime :1;
 
     /**
      * Write AAIP as extension according to SUSP 1.10 rather than SUSP 1.12.
@@ -430,7 +597,7 @@ struct isoburn_imgen_opts {
     int fifo_size;
 
 
-    /** Output value: Block address of session start as evaluated from media
+    /** Output value: Block address of session start as evaluated from medium
                       and other options by libisoburn and libburn.
         If <0 : Invalid
         If >=0: Valid block number. Block size is always 2 KiB.
@@ -443,6 +610,97 @@ struct isoburn_imgen_opts {
         If >=16: Valid block number. Block size is always 2 KiB.
     */
     int data_start_lba;
+
+    /**
+     * If not empty: Parameters "name" and "timestamp" for a scdbackup stream
+     * checksum tag. See scdbackup/README appendix VERIFY.
+     * It makes sense only for single session images which start at LBA 0.
+     * Such a tag may be part of a libisofs checksum tag block after the
+     * session tag line. It then covers the whole session up to its own start
+     * position.
+     * If scdbackup_tag_written is not NULL then it is a pointer to an
+     * application provided array with at least 512 characters. The effectively
+     * written scdbackup tag will be copied to this memory location.
+     */
+    char scdbackup_tag_name[81];
+    char scdbackup_tag_time[19];
+    char *scdbackup_tag_written;
+
+
+    /* Content of an embedded boot image. Valid if not NULL.
+     * In that case it must point to a memory buffer at least 32 kB.
+     */
+    char *system_area_data;
+    /*
+     * bit0= make bytes 446 - 512 of the system area a partition
+     *       table which reserves partition 1 from byte 63*512 to the
+     *       end of the ISO image. Assumed are 63 secs/hed, 255 head/cyl.
+     *       (GRUB protective msdos label.)
+     *       This works with and without system_area_data.
+     */
+    int system_area_options;
+
+    /* User settable PVD time stamps */
+    time_t vol_creation_time;
+    time_t vol_modification_time;
+    time_t vol_expiration_time;
+    time_t vol_effective_time;
+    /* To eventually override vol_modification_time by unconverted string
+       and timezone 0 */
+    char vol_uuid[17];
+
+    /* The number of unclaimed 2K blocks before start of partition 1 as of
+       the MBR in system area. If not 0 this will cause double volume
+       descriptor sets and double tree.
+    */
+    uint32_t partition_offset;
+    /* Partition table parameter: 1 to 63, 0= disabled/default */
+    int partition_secs_per_head; 
+    /* 1 to 255, 0= disabled/default */
+    int partition_heads_per_cyl;
+
+    /* Parameters and state of Jigdo Template Export environment.
+    */
+    void *libjte_handle;
+
+    /* A trailing padding of zero bytes which belongs to the image
+    */
+    uint32_t tail_blocks;
+
+    /* Disk file paths of content of PreP partition and EFI system partition */
+    char *prep_partition;
+    char *efi_boot_partition;
+
+    /* Eventual disk file paths of prepared images which shall be appended
+       after the ISO image and described by partiton table entries in a MBR.
+    */
+    char *appended_partitions[Libisoburn_max_appended_partitionS];
+    uint8_t appended_part_types[Libisoburn_max_appended_partitionS];
+
+    /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
+     */
+    char ascii_disc_label[Libisoburn_disc_label_sizE];
+
+    /* HFS+ image serial number.
+     * 00...00 means that it shall be generated by libisofs.
+     */
+    uint8_t hfsp_serial_number[8];
+
+    /* Allocation block size of HFS+ : 0= auto , 512, or 2048
+     */
+    int hfsp_block_size;
+
+    /* Block size of and in APM : 0= auto , 512, or 2048
+     */
+    int apm_block_size;
+
+    /* Write mode for optical media:
+     *   0 = auto
+     *   1 = TAO, Incremental, no RESERVE TRACK
+     *  -1 = SAO, DAO, RESERVE TRACK
+     */
+    int do_tao;
+
 };
 
 
@@ -473,11 +731,14 @@ struct isoburn_imgen_opts {
    Libisoburn_nwa_alignemenT it should be possible to predict the start
    of the next session header.
    The LBA 0 header is written by isoburn_activate_session() already
-   with the first session. So the media is mountable.
+   with the first session. So the medium is mountable.
    A problem arises with DVD-RW in Intermediate State. They cannot be
    written by random access before they were written sequentially.
    In this case, no copy of the session  1 header is maintained and no TOC
    will be possible. Thus writing begins sequentially at LBA 0.
+
+   IMPORTANT: This macro gives the minimal size of an image header. 
+              It has to be enlarged by the eventual partition offset.
 */
 #define Libisoburn_overwriteable_starT \
                                    ((off_t) (Libisoburn_target_head_sizE/2048))
