@@ -565,6 +565,22 @@ ex:;
 }
 
 
+/* Option -rockridge "on"|"off" */
+int Xorriso_option_rockridge(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "off")==0)
+   xorriso->do_rockridge= 0;
+ else if(strcmp(mode, "on")==0)
+   xorriso->do_rockridge= 1;
+ else {
+   sprintf(xorriso->info_text, "-rockridge: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -rollback */
 /* @param flag bit0= do not -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -1906,6 +1922,26 @@ ex:;
 }
 
 
+/* Command -write_type */
+int Xorriso_option_write_type(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "auto") == 0)
+   xorriso->do_tao = 0;
+ else if(strcmp(mode, "tao") == 0 || strcmp(mode, "TAO") == 0) 
+   xorriso->do_tao = 1;
+ else if(strcmp(mode, "sao") == 0 || strcmp(mode, "SAO") == 0 ||
+         strcmp(mode, "dao") == 0 || strcmp(mode, "DAO") == 0 ||
+         strcmp(mode, "sao/dao") == 0 || strcmp(mode, "SAO/DAO") == 0 ) 
+   xorriso->do_tao = -1;
+ else {
+   sprintf(xorriso->info_text, "-write_type: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -xattr "on"|"off" */
 int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag)
 {
